# -*- python -*-
from __future__ import print_function
import re

class BeautifulTbl(object):
  def __init__(self, **kwargs):
    self.__gap      = kwargs.get("gap",        2)
    self.__column   = kwargs.get("column",     0)
    self.__wrapped  = kwargs.get("wrapped",    False)
    self.__justify  = kwargs.get("justify",    False) or "".join(kwargs.get("justifyT",   []))
    self.__tbl      = self.__build_tbl(kwargs.get("tbl",None))

  def __build_tbl(self,tblIn):
    columnCnt = []
    tbl       = []
    justify   = self.__justify
    maxnc     = 0
    gap       = self.__gap

    for irow, row in enumerate(tblIn):
      for icol, v in enumerate(row):
        v = str(v)
        if (icol == maxnc):
          columnCnt.append(0)
          maxnc += 1
        columnCnt[icol] = max(len(v), columnCnt[icol])

    # build justifyT
    nc = len(justify)
    justifyT = []
    for icol in range(nc):
      justifyT.append((justify[icol].lower() == "r") and "r" or "l")

    for i in range(nc, maxnc):
      justifyT.append("l")

    for irow, row in enumerate(tblIn):
      b    = []
      numC = len(row)
      for icol, v in enumerate(row):
        v = str(v)
        if (numC == 1):
          b.append(v)
        else:
          nspaces = columnCnt[icol] - len(v)
          if (justifyT[icol] == "l"):
            b.append(v + " "*(nspaces+gap))
          else:
            b.append(" "*nspaces + v + " "*gap)
      tbl.append(b)
    self.__columnCnt = columnCnt
    self.__maxnc     = maxnc
    self.__justifyT  = justifyT
    return tbl

  def build_tbl(self):

    width      = 0
    column     = self.__column - 1
    gap        = self.__gap
    simple     = True
    columnCnt  = self.__columnCnt
    justifyT   = self.__justifyT

    if (self.__wrapped and self.__column > 0):
      for icol in range(len(columnCnt)-1):
        width += columnCnt[icol] + gap
      last = columnCnt[-1]
      simple = (width > column - 10) or (width + last < column)

    tbl = self.__tbl

    if (len(tbl) < 1):
      return ""

    a = []
    if (simple):
      for row in tbl:
        row[-1] = row[-1].rstrip()
        a.append("".join(row))
      return "\n".join(a)

    # If here then the last column must be wrapped. It removes any
    # trailing spaces.  Note that the last column is the only column
    # that is word wrapped.  Any short rows are copied straight
    # across.

    gap    = column - width
    fill   = " "*width

    maxnc  = self.__maxnc
    maxnc1 = maxnc - 1

    for irow, row in enumerate(tbl):
      aa  = []
      nc  = len(row)
      nc1 = min(nc, maxnc1)

      for i in range(nc1):
        aa.append(row[i])

      # Now word wrap last column
      aaa = []
      if (nc == maxnc):
        icnt = width
        s    = row[-1]
        iter_matches = re.finditer(r'\w+', s)
        for match in iter_matches:
          word = match.group(0)
          wlen = len(word)+1
          if   (word == ""):
            wlen = 0
          elif (icnt + wlen < column or wlen > gap ):
            aaa.append(word + " ")
          else:
            aaa[-1] = aaa[-1].rstrip()
            line   = "".join(aaa)
            if (justifyT[-1] == "r"):
                aa.append(" "*(gap - len(line)))
            aa.append(line)
            aa.append("\n")
            a.append("".join(aa))
            aaa = []
            aa  = []
            aa.append(fill)
            aaa.append(word + " ")
            icnt = width
          icnt += wlen
      aaa[-1] = aaa[-1].rstrip()
      line    = "".join(aaa)
      if (justifyT[-1] == "r"):
        aa.append(" "*(gap - len(line)))
      aa.append(line)
      aa.append("\n")
      a.append("".join(aa))
    return "".join(a)


def main():

  a = [
      ["a23456", "b23456789 123456789 123456", "c0_456789 c1_456789 c2_456789 c3_456789 c4_456789 c5_456 c6_4567"],
      ["A23",    "B23456789 12345",            "C0_456789 C1_456789 C2_456789 C3_45678"],
      ]

  b = [
      ["abc as", "d a alsdkjf lkjasdf alsdjf", "laskdjfasjkd alsdkj alksdj asdlkfj asdlk aalsdkjf pasd f"],
      ["def", "ghialksdflaksdf", "la laksjd alskd laksjd asdlkfj asdpsdf"],
      ]


  bt = BeautifulTbl(tbl=a,column=79, wrapped=True, justify = "rrr")
  s  = bt.build_tbl()
  print ("         1         2         3         4         5         6         7         8")
  print ("123456789 "*8)

  print (s)

if ( __name__ == '__main__'): main()







# -*- python -*-
#-----------------------------------------------------------------------
# XALT: A tool that tracks users jobs and environments on a cluster.
# Copyright (C) 2013-2014 University of Texas at Austin
# Copyright (C) 2013-2014 University of Tennessee
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of
# the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser  General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#-----------------------------------------------------------------------

from __future__ import print_function, division
import os, sys, re

def getTerminalSize():
  """
  Finds the terminal size if possible otherwise it assumes 25, 80
  @returns: lines  the number of rows
  @returns: cols   the number of columns
  """
  import os, struct
  def ioctl_GWINSZ(fd):
      import fcntl, termios
      return struct.unpack("hh", fcntl.ioctl(fd, termios.TIOCGWINSZ, "1234"))
  # try stdin, stdout, stderr
  if (not sys.stdout.isatty()):
    return (25, 80)

  fd = 1
  try:
    return ioctl_GWINSZ(fd)
  except:
    pass
  # try os.ctermid()
  try:
    fd = os.open(os.ctermid(), os.O_RDONLY)
    try:
      return ioctl_GWINSZ(fd)
    finally:
      os.close(fd)
  except:
    pass
  # try `stty size`
  try:
    return tuple(int(x) for x in os.popen("stty size", "r").read().split())
  except:
    pass
  # try environment variables
  try:
    return tuple(int(os.getenv(var)) for var in ("LINES", "COLUMNS"))
  except:
    pass
  # i give up. return default.
  return (25, 80)

class ProgressBar(object):
  """ A progress bar display class """

  def __init__(self, termWidth=None, barWidth=None, maxVal=None, ttyOnly=False, fd=sys.stderr):
    """
    Ctor that figures out the range of the progress bar.
    @param termWidth: The terminal width
    @param barWidth:  The number of character that make up the bar
    @param maxVal:    The maximum value (required)
    @param ttyOnly:   If true then only print progress bar element if connected to a terminal.
    @param fd:        The output stream.
    """
    if (not maxVal):
      ValueError('Must specify maxVal')

    self.__maxVal = maxVal
    if (not barWidth):
      barWidth = termWidth
      if (not barWidth):
        r, barWidth = getTerminalSize()
        if (barWidth < 40):
          barWidth = 40

    barWidth = min(barWidth, self.__maxVal)

    self.__active   = True
    if (ttyOnly and not sys.stdout.isatty()):
      self.__active = False



    self.__fd       = fd
    self.__barWidth = barWidth

    self.__unit     = 100/self.__barWidth
    self.__fence    = self.__unit
    self.__mark     = 10
    self.__count    = -1
    self.__symbolT  = [ '+', '+', '+', '+', '|', '+', '+', '+', '+', '|' ]


  def update(self, i):
    """
    Update progress bar.
    @param i:  input value
    """

    if (not self.__active):
      return

    j = 100*i//self.__maxVal
    k = 100*(i+1)//self.__maxVal

    #print("i: ", i, "j: ",j,"k:",k, "fence: ", self.__fence,"mark:",self.__mark)

    if (j >= self.__fence):
      symbol = "-"
      if (( j <= self.__mark and k >= self.__mark) or
          ( j == k and j == self.__mark)         or
          (self.__fence > self.__mark)):

        self.__count +=  1
        self.__mark  += 10
        symbol = self.__symbolT[self.__count]
      self.__fd.write(symbol)
      self.__fence += self.__unit
  def fini(self):
    """ Finish progress bar output. """
    if (not self.__active):
      return
    self.__fd.write("\n")

def main():
  """
  Test program for progress bar.
  """

  num = 200
  pbar = ProgressBar(maxVal=num, ttyOnly=False)
  for i in xrange(num):
    pbar.update(i+1)
  pbar.fini()

if ( __name__ == '__main__'): main()




from __future__ import print_function
from time       import sleep
import os, sys, re, base64, time, json, traceback
dirNm, execName = os.path.split(os.path.realpath(sys.argv[0]))
sys.path.append(os.path.realpath(dirNm))

try:
  import configparser
except:
  import ConfigParser as configparser

import MySQLdb, getpass
import warnings, inspect
from BeautifulTbl import BeautifulTbl
warnings.filterwarnings("ignore", "Unknown table.*")
def __LINE__():
    try:
        raise Exception
    except:
        return sys.exc_info()[2].tb_frame.f_back.f_lineno

def __FILE__():
    return inspect.currentframe().f_code.co_filename

#print ("file: '%s', line: %d" % (__FILE__(), __LINE__()), file=sys.stderr)

def convertToInt(s):
  """
  Convert to string to int.  Protect against bad input.
  @param s: Input string
  @return: integer value.  If bad return 0.
  """
  try:
    value = int(s)
  except ValueError:
    value = 0
  return value

class LMODdb(object):
  """
  This LMODdb class opens the Lmod database and is responsible for
  all the database interactions.
  """
  s_build_userT   = True
  userT           = {}
  s_build_moduleT = True
  moduleT         = {}

  def __init__(self, confFn):
    """ Initialize the class and save the db config file. """
    self.__host   = None
    self.__user   = None
    self.__passwd = None
    self.__db     = None
    self.__conn   = None
    self.__confFn = confFn

  def __readFromUser(self):
    """ Ask user for database access info. (private) """

    self.__host   = raw_input("Database host:")
    self.__user   = raw_input("Database user:")
    self.__passwd = getpass.getpass("Database pass:")
    self.__db     = raw_input("Database name:")

  def __readConfig(self):
    """ Read database access info from config file. (private)"""
    confFn = self.__confFn
    try:
      config=configparser.ConfigParser()
      config.read(confFn)
      self.__host    = config.get("MYSQL","HOST")
      self.__user    = config.get("MYSQL","USER")
      self.__passwd  = base64.b64decode(config.get("MYSQL","PASSWD").encode()).decode()
      self.__db      = config.get("MYSQL","DB")
    except ConfigParser.NoOptionError as err:
      sys.stderr.write("\nCannot parse the config file\n")
      sys.stderr.write("Switch to user input mode...\n\n")
      self.__readFromUser()

  def connect(self, databaseName = None):
    """
    Public interface to connect to DB.
    @param db:  If this exists it will be used.

    """
    if(os.path.exists(self.__confFn)):
      self.__readConfig()
    else:
      self.__readFromUser()

    n = 200
    for i in range(0,n+1):
      try:
        self.__conn = MySQLdb.connect (self.__host,self.__user,self.__passwd)
        if (databaseName):
          cursor = self.__conn.cursor()

          # If MySQL version < 4.1, comment out the line below
          cursor.execute("SET SQL_MODE=\"NO_AUTO_VALUE_ON_ZERO\"")
          cursor.execute("USE "+self.db())
        break


      except MySQLdb.Error as e:
        if (i < n):
          sleep(i*0.1)
          print ("failed to connect trying again: ",i)
          pass
        else:
          print ("LMODdb(%d): Error %d: %s" % (i, e.args[0], e.args[1]), file=sys.stderr)
          raise
    return self.__conn


  def db(self):
    """ Return name of db"""
    return self.__db


  def dump_db(self, startDate, endDate):

    userA   = None
    moduleA = None

    query = ""
    try:
      conn   = self.connect()
      cursor = conn.cursor()
      query  = "USE "+self.db()
      conn.query(query)
      
      # extract user names from userT table.

      query = "SELECT max(user_id) from userT"
      cursor.execute(query)
      sz    = cursor.fetchone()[0]
      userA = [ None ] * (sz+1)

      query = "SELECT user_id, user from userT"
      cursor.execute(query)
      numRows = cursor.rowcount
      for i in range(numRows):
        row            = cursor.fetchone()
        user_id        = row[0]
        user           = row[1]
        userA[user_id] = user

      moduleA = []

      # extract moduleT table
      query = "SELECT max(mod_id) from moduleT"
      cursor.execute(query)
      sz      = cursor.fetchone()[0]
      moduleA = [ None ] * (sz+1)

      query = "SELECT mod_id, path, module, syshost from moduleT"
      cursor.execute(query)
      numRows = cursor.rowcount
      for i in range(numRows):
        row             = cursor.fetchone()
        mod_id          = row[0]
        path            = row[1]
        module          = row[2]
        syshost         = row[3]
        moduleA[mod_id] = {'path' : path, 'module' : module, 'syshost' : syshost}


      # extract join_user_module table:
      query = "SELECT user_id, mod_id, UNIX_TIMESTAMP(date) from join_user_module where date >= %s AND date < %s"
      cursor.execute(query, (startDate, endDate))
      numRows = cursor.rowcount
      for i in range(numRows):
        row     = cursor.fetchone()
        user    = userA[int(row[0])]
        modT    = moduleA[int(row[1])]
        date    = str(row[2])
        t       = {'user': user, 'path': modT['path'], 'module' : modT['module'], 'syshost': modT['syshost'], 'date' : date }
        print(json.dumps(t))

    except Exception as e:
      print("dump_db(): ",e)
      sys.exit(1)


  def data_to_db(self, debug, count, dataT, line):
    """
    Store data into database.
    @param dataT: The data table.
    """

    query = ""
    try:
      if (debug): print("  --> Trying to connect to database")
      conn   = self.connect()
      cursor = conn.cursor()
      query  = "USE "+self.db()
      conn.query(query)
      query  = "START TRANSACTION"
      conn.query(query)

      ############################################################
      # build userT if necessary.

      if (LMODdb.s_build_userT):
        if (debug): print("  --> Build userT")
        LMODdb.s_build_userT = False

        query = "SELECT user_id, user from userT"
        cursor.execute(query)

        numRows = cursor.rowcount
        for i in range(numRows):
          row = cursor.fetchone()
          user_id = int(row[0])
          user    = row[1]
          LMODdb.userT[user] = user_id
          
      ############################################################
      # Get user_id
      user    = dataT['user']
      user_id = LMODdb.userT.get(user)

      if (user_id == None):
        
        try:
          if (debug): print("  --> Insert into userT")
          query = "INSERT INTO userT VALUES(NULL, %s)"
          cursor.execute(query,[user])
          user_id = cursor.lastrowid
        except Exception as e:
          query = "SELECT user_id FROM userT WHERE user = %s"
          cursor.execute(query,[user])
          if (cursor.rowcount > 0):
            user_id = int(cursor.fetchone()[0])
          else:
            print("Failed to insert \"%s\" into userT, Aborting!" % user)
            print(traceback.format_exc())
            sys.exit(1)

        LMODdb.userT[user] = user_id
       
      #############################################################
      # build moduleT

      if (LMODdb.s_build_moduleT):
        if (debug): print("  --> build moduleT")
        LMODdb.s_build_moduleT = False

        query = "SELECT mod_id, path, module, syshost FROM moduleT"
        cursor.execute(query)
        numRows = cursor.rowcount

        for i in range(numRows):
          row     = cursor.fetchone()
          mod_id  = row[0]
          path    = row[1]
          module  = row[2]
          syshost = row[3]
          LMODdb.moduleT[(syshost,path)] =  mod_id
          
      ############################################################
      # get mod_id
      
      syshost = dataT['syshost']
      path    = dataT['path']
      key     = (syshost, path)
      mod_id  = LMODdb.moduleT.get(key)
      module  = dataT['module']
      if (mod_id == None):

        if (debug): print("  --> get mod_id")
        try:
          query = "INSERT INTO moduleT VALUES(NULL, %s, %s, %s)"
          cursor.execute(query,(path, module, syshost))
          mod_id = cursor.lastrowid
        except Exception as e:
          query = "SELECT mod_id FROM moduleT WHERE syshost = %s and path = %s "
          cursor.execute(query,(syshost, path))
          if (cursor.rowcount > 0):
            mod_id = int(cursor.fetchone()[0])
          else:
            print(traceback.format_exc())
            print("Failed to insert (\"%s\",\"%s\") moduleT, Aborting!" % (syshost, path))
            sys.exit(1)

        LMODdb.moduleT[key] = mod_id
        
        
      ############################################################
      # Insert into join_user_module table
      
      if (debug): print("  --> Insert into join_user_module Table for user:", user,", line:",count+1,", module:",module)
      dateTm = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(float(dataT['time'])))
      query  = "INSERT into join_user_module VALUES(NULL, %s, %s, %s) "
      cursor.execute(query,(user_id, mod_id, dateTm))
      if (debug): print("  --> Done")

      ##################################################################
      # Step 4: Commit everything to db.

      query = "COMMIT"
      conn.query(query)
      conn.close()

    except Exception as e:
      print("query: ",query)
      print("line: ",line)
      for k in dataT:
        print(" ",k,":",dataT[k])
      print(traceback.format_exc())
      print("data_to_db(): ",e)
      sys.exit(1)

  def counts(self, sqlPattern, syshost, startDate, endDate, allmodulesFn):
    query = ""
    try:
      conn   = self.connect()
      cursor = conn.cursor()
      query  = "USE "+self.db()
      conn.query(query)

      dateTest = ""
      if (startDate != "unknown"):
        dateTest = " and t2.date >= '" + startDate + "'"

      if (endDate != "unknown"):
        dateTest = dateTest + " and t2.date < '" + endDate + "'"

      if (sqlPattern == "") :
        sqlPattern == "%"

      query = "SELECT t1.path, t1.syshost, count(distinct(t2.user_id)) as c3 from moduleT as t1, "  +\
              "join_user_module as t2 where t1.path like %s and t1.mod_id = t2.mod_id " +\
              "and t1.syshost like %s " + dateTest + " group by t2.mod_id order by c3 desc"

      cursor.execute(query,( sqlPattern, syshost))
      numRows = cursor.rowcount

      resultT = {}
      sT = {}

      if (allmodulesFn):
        with open(allmodulesFn) as fp:
          lineA = fp.readlines()
          for moduleNm in lineA:
            moduleNm          = moduleNm.strip()
            resultT[moduleNm] = { 'syshost' : syshost, 'nUsers' : 0 }
            sT[moduleNm]      = 0
      

      for i in range(numRows):
        row = cursor.fetchone()
        moduleNm = row[0]
        resultT[moduleNm] = { 'syshost' : syshost, 'nUsers' : row[2] }
        sT[moduleNm]      = row[2]

      resultA = []

      resultA.append(["Module path", "Syshost", "Distinct Users" ])
      resultA.append(["-----------", "-------", "--------------"])
      sT_view = [ (v,k) for k,v in sT.items() ]
      sT_view.sort(reverse=True)
      for v,k in sT_view:
        resultA.append([k, resultT[k]['syshost'], resultT[k]['nUsers'] ])

      conn.close()

      return resultA


    except Exception as e:
      print("counts(): ",e)
      sys.exit(1)

  def numtimes(self, sqlPattern, syshost, startDate, endDate):
    query = ""
    try:
      conn   = self.connect()
      cursor = conn.cursor()
      query  = "USE "+self.db()
      conn.query(query)

      dateTest = ""
      if (startDate != "unknown"):
        dateTest = " and t2.date >= '" + startDate + "'"

      if (endDate != "unknown"):
        dateTest = dateTest + " and t2.date < '" + endDate + "'"

      if (sqlPattern == "") :
        sqlPattern == "%"

      query = "SELECT t1.path, t1.syshost, count(distinct(t2.mod_id)) as c3 from moduleT as t1, "  +\
              "join_user_module as t2 where t1.path like %s and t1.syshost like %s "      +\
               dateTest + " group by t1.path order by c3 desc"

      cursor.execute(query,( sqlPattern, syshost))
      numRows = cursor.rowcount

      resultA = []

      resultA.append(["Module path", "Syshost", "Number of Times" ])
      resultA.append(["-----------", "-------", "---------------"])
      for i in range(numRows):
        row = cursor.fetchone()
        resultA.append([row[0],syshost,row[2]])

      conn.close()

      return resultA


    except Exception as e:
      print("numtimes(): ",e)
      sys.exit(1)

  def usernames(self, sqlPattern, syshost, startDate, endDate):
    query = ""
    try:
      conn   = self.connect()
      cursor = conn.cursor()
      query  = "USE "+self.db()
      conn.query(query)

      dateTest = ""
      if (startDate != "unknown"):
        dateTest = " and t2.date >= '" + startDate + "'"

      if (endDate   != "unknown"):
        dateTest = dateTest + " and t2.date < '" + endDate + "'"

      if (sqlPattern == "") :
        sqlPattern == "%"

      query =  "SELECT t1.path, t1.syshost, t3.user as c2 from moduleT as t1, join_user_module "       +\
               "as t2, userT as t3 where t1.path like  %s  and t1.mod_id = t2.mod_id "     +\
               "and t1.syshost like  %s " + dateTest +" and t3.user_id = t2.user_id "      +\
               "group by c2 order by c2"

      cursor.execute(query, ( sqlPattern, syshost))
      numRows = cursor.rowcount

      resultA = []
      resultA.append(["Module path", "Syshost", "User Name"])
      resultA.append(["-----------", "-------", "---------"])


      for i in range(numRows):
        row = cursor.fetchone()
        resultA.append([row[0],syshost,row[2]])

      conn.close()

      return resultA


    except Exception as e:
      print("usernames(): ",e)
      sys.exit(1)


  def modules_used_by(self, username, syshost, startDate, endDate):
    query = ""
    try:
      conn   = self.connect()
      cursor = conn.cursor()
      query  = "USE "+self.db()
      conn.query(query)

      dateTest = ""
      if (startDate != "unknown"):
        dateTest = " and t2.date >= '" + startDate + "'"

      if (endDate != "unknown"):
        dateTest = dateTest + " and t2.date < '" + endDate + "'"

      query =  "SELECT t1.path as c1, t1.syshost, t3.user as c2 from moduleT as t1, join_user_module "   +\
               "as t2, userT as t3 where t3.user =  %s  and t1.mod_id = t2.mod_id "          +\
               "and t1.syshost like %s "+ dateTest + " and t3.user_id = t2.user_id "         +\
               "group by c1 order by c1"

      cursor.execute(query, ( username, syshost ))
      numRows = cursor.rowcount

      resultA = []
      resultA.append(["Module path", "Syshost", "User Name"])
      resultA.append(["-----------", "-------", "---------"])


      for i in range(numRows):
        row = cursor.fetchone()
        resultA.append([row[0],syshost,row[2]])

      conn.close()

      return resultA


    except Exception as e:
      print("modules_used_by(): ",e)
      sys.exit(1)



#!/usr/bin/env python
# -*- python -*-

from __future__ import print_function
import os, sys, re, MySQLdb, time

dirNm, execName = os.path.split(os.path.realpath(sys.argv[0]))
sys.path.append(os.path.realpath(dirNm))

from LMODdb     import LMODdb
import argparse

def dbConfigFn(dbname):
  """
  Build config file name from dbname.
  @param dbname: db name
  """
  return dbname + "_db.conf"

def strDate2dA(s):
  dA = re.split(r'[-_/.]', s)
  dA = [ int(dA[0]), int(dA[1]) ]
  return dA

def add_month(dA):
  dA[1] += 1
  if (dA[1] > 12):
    dA[0] += 1
    dA[1]  = 1

  return dA

def substract_month(dA):
  dA[1] -= 1
  if (dA[1] < 1):
    dA[0] -= 1
    dA[1]  = 12

  return dA

  
class CmdLineOptions(object):
  """ Command line Options class """

  def __init__(self):
    """ Empty Ctor """
    pass

  def execute(self):
    """ Specify command line arguments and parse the command line"""
    dA     = add_month(strDate2dA(time.strftime('%Y-%m-%d', time.localtime(time.time()))))
    partNm = "%04d-%02d" % (dA[0], dA[1])

    parser = argparse.ArgumentParser()
    parser.add_argument("--dbname",    dest='dbname',    action="store",      default = "lmod", help="lmod")
    parser.add_argument("--startPart", dest='firstPart', action="store",      default = partNm, help="first partition date")
    args = parser.parse_args()
    return args


def main():
  """
  This program creates the Database used by Lmod.
  """

  args     = CmdLineOptions().execute()
  configFn = dbConfigFn(args.dbname)

  if (not os.path.isfile(configFn)):
    dirNm, exe = os.path.split(sys.argv[0])
    fn         = os.path.join(dirNm, configFn)
    if (os.path.isfile(fn)):
      configFn = fn
    else:
      configFn = os.path.abspath(os.path.join(dirNm, "../site", configFn))

  lmod = LMODdb(configFn)

  try:
    conn   = lmod.connect()
    cursor = conn.cursor()
    idx    = 1

    cursor.execute("USE "+lmod.db())

    print("(%d) create join_link_object table" % idx); idx += 1

    sqlCommands = """
       create procedure CreateDataPartition (newPartValue DATETIME, tbName VARCHAR(50))
       begin
       DECLARE keepStmt VARCHAR(2000) DEFAULT @stmt;
       SET @stmt = CONCAT('ALTER TABLE ', tbName ,' ADD PARTITION (PARTITION p',
                            DATE_FORMAT(newPartValue, '%Y_%m'),
                            ' VALUES LESS THAN (TO_DAYS(\\'',
                            DATE_FORMAT(newPartValue, '%Y-%m-01'),
                            '\\')))');
       PREPARE pStmt FROM @stmt;
       execute pStmt;
       DEALLOCATE PREPARE pStmt;
       set @stmt = keepStmt;
       END 
    """


    cursor.execute(sqlCommands)
    print("(%d) Create stored procedure CreateDataPartition" % idx); idx += 1

    sqlCommands = """
       CREATE EVENT eventCreatePartition
       ON SCHEDULE EVERY 1 MONTH
       STARTS '2016-02-01 00:00:00'
       DO
       BEGIN
        call CreateDataPartition(NOW() + interval 1 MONTH,'join_user_module');
       END 
    """
    cursor.execute(sqlCommands)
    print("(%d) Create event eventCreatePartition" % idx); idx += 1

    dA  = strDate2dA(args.firstPart)
    dA  = add_month(dA)
    now = time.strftime('%Y-%m-%d', time.localtime(time.time()))
    eA  = add_month(strDate2dA(now))

    while ((dA[0] < eA[0]) or (dA[0] == eA[0] and dA[1] <= eA[1])):
      partNm  = "p%04d_%02d"   % (dA[0], dA[1])
      dateStr = "%04d-%02d-01" % (dA[0], dA[1])
      query   = "ALTER TABLE join_user_module ADD PARTITION (PARTITION " +\
               partNm + " VALUES LESS THAN (TO_DAYS('"+dateStr+"')))"

      print("query:",query)
      cursor.execute(query)

      dA = add_month(dA)

    cursor.close()

  except  MySQLdb.Error, e:
        print ("Error %d: %s" % (e.args[0], e.args[1]))
        sys.exit (1)

if ( __name__ == '__main__'): main()



#!/bin/sh
# -*- python -*-

################################################################################
# This file is python 2/3 bilingual. 
# The line """:" starts a comment in python and is a no-op in shell.
""":"
# Shell code to find and run a suitable python interpreter.
for cmd in python3 python python2; do
   command -v > /dev/null $cmd && exec $cmd $0 "$@"
done

echo "Error: Could not find a valid python interpreter --> exiting!" >&2
exit 2
":""" # this line ends the python comment and is a no-op in shell.
################################################################################

from __future__ import print_function
import os, sys, re, MySQLdb, time

dirNm, execName = os.path.split(os.path.realpath(sys.argv[0]))
sys.path.append(os.path.realpath(dirNm))

from LMODdb     import LMODdb
import argparse

def dbConfigFn(dbname):
  """
  Build config file name from dbname.
  @param dbname: db name
  """
  return dbname + "_db.conf"

class CmdLineOptions(object):
  """ Command line Options class """

  def __init__(self):
    """ Empty Ctor """
    pass

  def execute(self):
    """ Specify command line arguments and parse the command line"""
    parser = argparse.ArgumentParser()
    parser.add_argument("--drop",        dest='drop',   action="store_true", default = False,     help="lmod")
    parser.add_argument("--confFn",      dest='confFn', action="store",      default = None,      help="lmod")
    parser.add_argument("--dbname",      dest='dbname', action="store",      default = "lmod",    help="lmod")
    args = parser.parse_args()
    return args


def main():
  """
  This program creates the Database used by Lmod.
  """

  args     = CmdLineOptions().execute()
  configFn = dbConfigFn(args.dbname)
  if (args.confFn):
    configFn = args.confFn

  if (not os.path.isfile(configFn)):
    dirNm, exe = os.path.split(sys.argv[0])
    fn         = os.path.join(dirNm, configFn)
    if (os.path.isfile(fn)):
      configFn = fn
    else:
      configFn = os.path.abspath(os.path.join(dirNm, "../site", configFn))

  lmod = LMODdb(configFn)

  try:
    conn   = lmod.connect()
    cursor = conn.cursor()

    # drop db if requested.
    if (args.drop):
      cursor.execute("DROP DATABASE IF EXISTS %s " % lmod.db())

    # If the database does not exist, create it, otherwise, switch to the database.
    cursor.execute("CREATE DATABASE IF NOT EXISTS %s DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci" % lmod.db())
    cursor.execute("USE "+lmod.db())

    idx = 1


    print("start")

    # 1
    cursor.execute("""
        CREATE TABLE `userT` (
          `user_id`       int(11) unsigned NOT NULL auto_increment,
          `user`          varchar(64)      NOT NULL,
          PRIMARY KEY  (`user_id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8  COLLATE=utf8_general_ci AUTO_INCREMENT=1
        """)
    print("(%d) create userT table" % idx); idx += 1

    # 2
    cursor.execute("""
        CREATE TABLE `moduleT` (
          `mod_id`        int(11) unsigned NOT NULL auto_increment,
          `path`          varchar(1024)    NOT NULL,
          `module`        varchar(64)      NOT NULL,
          `syshost`       varchar(32)      NOT NULL,
          PRIMARY KEY  (`mod_id`),
          INDEX  `thekey` (`path`(128), `syshost`)

        ) ENGINE=InnoDB DEFAULT CHARSET=utf8  COLLATE=utf8_general_ci AUTO_INCREMENT=1
        """)
    print("(%d) create moduleT table" % idx ); idx += 1;


    # 3
    cursor.execute("""
        CREATE TABLE `join_user_module` (
          `join_id`       int(11) unsigned NOT NULL auto_increment,
          `user_id`       int(11) unsigned NOT NULL,
          `mod_id`        int(11) unsigned NOT NULL,
          `date`          DATETIME         NOT NULL,
          PRIMARY KEY (`join_id`, `date`),
          INDEX `index_date` (`date`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8  COLLATE=utf8_general_ci AUTO_INCREMENT=1
        """)
    print("(%d) create join_link_object table" % idx); idx += 1

    cursor.close()
  except  MySQLdb.Error as e:
    print ("Error %d: %s" % (e.args[0], e.args[1]))
    sys.exit (1)

if ( __name__ == '__main__'): main()
